// VirtualTreeDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TVItemData.h"
#include "TVItemDataMgr.h"
#include "VirtualTree.h"
#include "VirtualTreeDlg.h"
#include ".\virtualtreedlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CVirtualTreeDlg dialog



CVirtualTreeDlg::CVirtualTreeDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CVirtualTreeDlg::IDD, pParent)
   , m_sStatus( _T("") )
   , m_uMaxCount( 20000 )
   , m_uDepth( 4 )
   , m_uMultiplier( 1 )
   , m_sDrive( _T("") )
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CVirtualTreeDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_TREE, m_ctlTree);
   DDX_Control(pDX, IDC_SPIN_MAX_COUNT, m_ctlMaxCountSpin);
   DDX_Control(pDX, IDC_SPIN_DEPTH, m_ctlDepthSpin);
   DDX_Control(pDX, IDC_SPIN_MULTIPLIER, m_ctlMultiplierSpin);
   DDX_Text(pDX, IDC_STATUS, m_sStatus);
   DDX_Control(pDX, ID_RETRIEVE, m_btnRetrieve);
   DDX_Control(pDX, IDCANCEL, m_btnCancel);
   DDX_Text(pDX, IDC_MAX_COUNT, m_uMaxCount);
   DDV_MinMaxUInt(pDX, m_uMaxCount, 1, 1000000);
   DDX_Text(pDX, IDC_DEPTH, m_uDepth);
   DDX_Text(pDX, IDC_MULTIPLIER, m_uMultiplier);
   DDX_Control(pDX, IDC_COMBO_DRIVES, m_ctlDrives);
   DDX_CBString(pDX, IDC_COMBO_DRIVES, m_sDrive);
}

BEGIN_MESSAGE_MAP(CVirtualTreeDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
   ON_BN_CLICKED(ID_RETRIEVE, OnBnClickedRetrieve)
   ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
   ON_NOTIFY(TVN_GETDISPINFO, IDC_TREE, OnTvnGetdispinfoTree)
END_MESSAGE_MAP()


// CVirtualTreeDlg message handlers

BOOL CVirtualTreeDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

   // Set the range on the spin controls
   m_ctlMaxCountSpin.SetRange32( 1, 100000 );
   m_ctlDepthSpin.SetRange32( 1, 20 );
   m_ctlMultiplierSpin.SetRange32( 1, 10 );

   // Fill the Drives combo box with local drive letter(s)
   FillDrives( );

   // Set the initial drive
   if( 0 < m_ctlDrives.GetCount( ) )
   {
      m_ctlDrives.SetCurSel( 0 );
   }
	
	return TRUE;
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CVirtualTreeDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CVirtualTreeDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

// Status text helper method
void CVirtualTreeDlg::SetStatusText( LPCTSTR szText )
{
    m_sStatus = szText;
    m_sStatus = _T(" ") + m_sStatus;
    UpdateData( FALSE );
}

void CVirtualTreeDlg::OnBnClickedRetrieve()
{
   m_btnRetrieve.EnableWindow( FALSE );
   UpdateData( TRUE );

   CWaitCursor wait;

   // Clear the tree ctrl items
   SetStatusText( _T("Clearing the tree control...") );
   m_ctlTree.LockWindowUpdate();
   m_ctlTree.DeleteAllItems();
   m_ctlTree.UpdateWindow();
   m_ctlTree.UnlockWindowUpdate();

   SetStatusText( _T("Retrieving directory structure...") );

   wait.Restore();
   m_TVItemDataMgr.Retrieve( m_sDrive, m_uMaxCount, m_uMultiplier, m_uDepth );

   SetStatusText( _T("Applying Multiplier...") );

   wait.Restore();
   m_TVItemDataMgr.ApplyMultiplier( );

   SetStatusText( _T("Populating tree control..") );

   wait.Restore();

   UINT uCount = 0;
   m_TVItemDataMgr.Populate( m_ctlTree, uCount );

   wait.Restore();
      
   m_ctlTree.UpdateWindow();

   // Enable the buttons
   m_btnCancel.EnableWindow( TRUE );
   m_btnRetrieve.EnableWindow( TRUE );

   CString sCount;
   sCount.Format( _T("Count: %d"), uCount );

   SetStatusText( sCount );
}

void CVirtualTreeDlg::OnBnClickedCancel()
{
   m_btnCancel.EnableWindow( FALSE );
   m_btnRetrieve.EnableWindow( FALSE );

   SetStatusText( _T("Clearing data - standby...") );

   CWaitCursor wait;

   // Clear the List control
   m_ctlTree.LockWindowUpdate();
   m_ctlTree.DeleteAllItems();
   m_ctlTree.UpdateWindow();
   m_ctlTree.UnlockWindowUpdate();

   wait.Restore();

   // Interrupt the secondary thread and delete the items
   m_TVItemDataMgr.Cancel();

   // Close the dialog
   OnCancel();
}

void CVirtualTreeDlg::FillDrives( )
{
   // NOTE: GetLogicalDriveStrings returns a MULTI-SZ string (i.e.
   // a string with embedded nulls).  As such we can't use std::string
   // or CString and must deal with the raw data buffer

   UINT uOffset = 0;
	DWORD dwBuffer = GetLogicalDriveStrings(0, (LPTSTR)NULL) + 1;
	PTCHAR lpBuffer = new TCHAR[dwBuffer + 1];

	if(GetLogicalDriveStrings( dwBuffer, (LPTSTR)lpBuffer ))
	{
		do
		{
			if(GetDriveType( lpBuffer + uOffset ) == DRIVE_FIXED)
			{
            // Insert the drive into the Drives Combobox
            // Strip off the backslash
            CString sDrive( lpBuffer + uOffset );
            sDrive.Delete( 2 );
            m_ctlDrives.AddString( sDrive );
			}
			uOffset += 4;
		}
		while( lstrlen( lpBuffer + uOffset ) );
	}
	delete [] lpBuffer;	
}
// Virtual Tree Control GetDispInfo handler
// This gets called each time the tree control needs to paint an item
// When we insert an item into the tree control, we store the pointer
// to the underlying TVItemData object (it gets stored in the TVItem's
// lParam member).  Since we link the tree item to the TVItemData object,
// to retrieve the display data, we just need to get the corresponding
// CTVItemData object for the item and call it's member to fill out the
// display
void CVirtualTreeDlg::OnTvnGetdispinfoTree(NMHDR *pNMHDR, LRESULT *pResult)
{
   LPNMTVDISPINFO pTVDispInfo = reinterpret_cast<LPNMTVDISPINFO>(pNMHDR);

   // Get the tree control item
   TVITEM* pItem = &(pTVDispInfo)->item;

   // Just interested in text notifications
   if( ( pItem->mask & TVIF_TEXT ) != TVIF_TEXT )
   {
      *pResult = 0;
      return;
   }

   // Retrieve the CLVItemData object associated with this item
	CTVItemData* pTVItemData = reinterpret_cast<CTVItemData*>(pItem->lParam);

   if( NULL != pTVItemData )
   {
      // Let the item method fill out the display data 
      pTVItemData->GetTreeViewItemInfo( pItem );
   }
   *pResult = 0;
}
